"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var AjaxWork;
(function (AjaxWork) {
    // The fuction bellow will be executed in worker
    // use it to execute the thread
    function Worker() {
        var _this = this;
        var locationOrigin = null;
        var sharedMethods = {
            orderStack: {},
            init: function (args) {
                _this.locationOrigin = args[0];
            },
            fetch: function (args) {
                var options = args[0];
                ajax(options);
            },
        };
        // ajax fetch ------------------------------
        var syncResponse = {};
        var syncRequestStack = [];
        function sendBack(options) {
            if (options.sync === true) {
                // store in the response store -----------------
                syncResponse[options.hash] = options;
                // loop in the requests ----------------
                while (syncRequestStack.length > 0) {
                    if (syncRequestStack[0].abort === true) {
                        execute("onAbort", [syncRequestStack[0]]);
                        syncRequestStack.shift();
                    }
                    else {
                        if (syncResponse[syncRequestStack[0].hash] === undefined) {
                            break;
                        }
                        else {
                            var r = syncResponse[syncRequestStack[0].hash];
                            execute("onDone", [r]);
                            syncRequestStack.shift();
                        }
                    }
                }
                // clean the syncResponse
                if (syncRequestStack.length === 0) {
                    syncResponse = {};
                    execute("clean", []);
                }
            }
            else {
                execute("onDone", [options]);
            }
        }
        function ajaxRequestStackPush(options) {
            if (options.sync === true) {
                for (var _i = 0, syncRequestStack_1 = syncRequestStack; _i < syncRequestStack_1.length; _i++) {
                    var i = syncRequestStack_1[_i];
                    if (options.id === i.id || options.hash === i.hash) {
                        i.abort = true;
                    }
                }
                syncRequestStack.push(options);
            }
        }
        function ajax(options) {
            var fetchReturn = {
                id: options.id,
                url: options.url,
                status: 404,
                returnType: options.returnType,
                statusText: "Error on Ajax-Worker!",
                data: null,
                headers: [],
                redirected: false,
                urlRedirected: null,
                errorMessage: null,
                error: false,
                sync: options.sync,
                hash: options.hash,
            };
            // sync ----------------------------
            // fetch ---------------------------
            // options.keepalive = false; // TODO: testing
            ajaxRequestStackPush(options);
            if (options.url === null) {
                console.error("url cannot be null");
                sendBack(fetchReturn);
                return;
            }
            var fetchPromise = fetch(options.url, options)
                .then(function (response) {
                fetchReturn.status = response.status;
                fetchReturn.statusText = response.statusText;
                fetchReturn.urlRedirected = response.url;
                fetchReturn.redirected = (fetchReturn.urlRedirected !== fetchReturn.url);
                fetchReturn.headers = response.headers;
                fetchReturn.headers = [];
                response.headers.forEach(function (value, key) {
                    fetchReturn.headers.push([key, value]);
                });
                var headerContentType = response.headers.get("content-type");
                var contentType = "text";
                if (headerContentType && (headerContentType.includes("application/json") || headerContentType.includes("text/json"))) {
                    contentType = "json";
                }
                else {
                    contentType = "text";
                }
                // -------------------------------
                if (contentType === "text") {
                    fetchReturn.returnType = "text";
                    return response.text();
                }
                else if (contentType === "json") {
                    fetchReturn.returnType = "json";
                    return response.json();
                }
            })
                .then(function (data) {
                fetchReturn.data = data;
                sendBack(fetchReturn);
            })
                .catch(function (error) {
                fetchReturn.errorMessage = error;
                fetchReturn.error = true;
                sendBack(fetchReturn);
            });
        }
        // same functions here
        function execute(actionName, args) {
            self.postMessage({
                method: actionName,
                args: args,
            });
        }
        self.onmessage = function (event) {
            var args = event.data.args;
            sharedMethods[event.data.method](args);
        };
    }
    AjaxWork.Worker = Worker;
})(AjaxWork = exports.AjaxWork || (exports.AjaxWork = {}));
exports.default = AjaxWork;
//# sourceMappingURL=ajax-work.js.map